;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;                       Disclaimer / Warranty
;;;
;;; The information contained herein is a shareware product
;;; soley for the benefit of licensed AutoCAD END USERS everywhere.  You
;;; are free to modify the information contained herein in any way, for
;;; ANY NON-COMMERCIAL use, provided such reproduction/disribution is NOT
;;; FOR PROFIT, and provided that with any such distribution, this NOTICE
;;; (or any portion thereof) is NOT REMOVED OR ALTERED from it's original
;;; content in ANY manner, shape, or form.
;;;
;;; Every effort has been made to make this program as complete, accurate
;;; and error free as possible.  But no warranty or fitness is implied.
;;;
;;; This program is provided on an "as is" basis.  The author shall have
;;; neither liability nor responsibility to any person or entity with 
;;; respect to any loss or damages arising from the use of this program 
;;; or any portion thereof.
;;;
;;; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;;;  IF YOU FIND THIS PROGRAM USEFUL, PLEASE SEND A DONATION OF $5.00 
;;;  TO THE ADDRESS LISTED BELOW.
;;; !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;
; Custom AutoLISP programming can be obtained by contacting:
;
;                          John W. Gibb
;                          14716 Trumbull Ave.
;                          Midlothian, IL.  60445
;
; Original version   4/25/94	John Gibb/Paladin (708) 597-8212
;				Compuserve #71530,3004
;
; -----------------------------------------------------------------------
;                             POLY.LSP
;
; This routine is to be used with any of the edit commands listed in
; cmdlist to select entities with an existing polyline.
; The polyline can have arcs in it but it will be treated as if it
; was all straight lines since it uses only the vertices of the entities.
;
; It will accept a string or integer argument.
; The integer arguments supported are: 0 for fence
;                                      1 for window polygon
;                                      2 for crossing polygon
;
; If the argument is a string the routine looks at the first character 
; in the string and applies the option to a polyline of the user's choice.
; Acceptable arguments are:   "C" or "CP" for Cross Polygon
;                             "W" or "WP" for Window Polygon
;                             "F" or "Fence" for a Fence
;
; If no legal integer or string is supplied the default is window polygon.
; If no item is picked the routine is dismissed.
; If an item that is not a polyline is chosen the routine gives a
; message and allows the user to try again.
;
; Fence polylines will only select objects crossing the polyline.
; Window polylines behave just like the WPolygon option for both closed
; & opened plines.
; Crossing polylines behave like the CPolygon option.
; Meaning it behaves as if both a closed and an open pline was closed.
; This is because the routine uses the CPolygon option behind the scenes.
;
; External routines used:  
; Internal routines used:  POLY_WORK
;                          POLY_PL_LIST
;                          POLY_ERROR
;
; Add the load for this routine to your ACAD.LSP routine for automatic
; loading.  Example:   (load "poly")
; To call by typing you must:
;                   Select an edit command
;                   At the Select objects prompt type (poly option)
; The option argument can be a string or integer as stated above.
;
; Add the calls for each option to a menu pulldown for easy use.
; Example:
;       [Poly Fence]^P(if (not poly)(load "poly"))(poly "F")
;       [Poly Window]^P(if (not poly)(load "poly"))(poly "WP")
;       [Poly Cross]^P(if (not poly)(load "poly"))(poly "CP")
;
;
; ************************************************************************
;
(defun poly (opt / cmdlist save-cmdecho save-error)
 (setq cmdlist '("ARRAY" "COPY" "MOVE" "MIRROR" "ERASE" "ROTATE" "CHPROP"
                 "SCALE" "STRETCH" "CHANGE" "EXPLODE" "SELECT"))
 (if (not (member (getvar "cmdnames") cmdlist))
  (princ "\nAn edit command must be active to use this option.")
  (progn
   (setq save-cmdecho (getvar "cmdecho")
         save-error *error*
         *error* poly_error)
   (setvar "cmdecho" 0)
   (if (= (type opt) 'STR)
    (setq opt (strcase (substr opt 1 1)))
   );end if
   (cond ((= opt "F")(poly_work "F"))
         ((= opt "C")(poly_work "CP"))
         ((= opt "W")(poly_work "WP"))
         ((= opt 0)(poly_work "F"))
         ((= opt 1)(poly_work "WP"))
         ((= opt 2)(poly_work "CP"))
         (T (poly_work "WP"))
   );end cond
   (setvar "cmdecho" save-cmdecho)
   (setq *error* save-error)
  );end progn
 );end if command active
 (princ)
)
(defun poly_work (opt / pt_list ent cnt closed)
 (if (setq ent (entsel "Select a poly line. "))
  (if (setq pt_list (poly_pl_list (car ent)))
   (progn
    (setq pt_list (reverse pt_list)
          cnt 0)
    (command opt)
    (repeat (length pt_list)
     (command (nth cnt pt_list))
     (setq cnt (1+ cnt))
    );end repeat
    (if closed (command (nth 0 pt_list)));end if
    (princ "\n")
    (command "")
   );end progn
   (poly_work opt)
  );end if
  (progn
;   (princ "\nMissed ... No entity picked.")
   nil
  );end progn
 );end if
 (princ)
)
(defun poly_pl_list (e / old pt_list bit)
  (if (= (cdr (assoc 0 (entget e))) "POLYLINE")  ;check for polyline
    (progn                                       ;if pline
                                                 ;loop until SEQEND reached
      (setq bit (cdr(assoc 70 (entget e))))
      (cond ((= 1 (logand bit 1))(setq closed T))
            ((= 32 (logand bit 32))(setq closed T))
            (T (setq closed nil))
      );end cond
      (while (/= (cdr (assoc 0 (entget e))) "SEQEND")
        (setq e (entnext e)                      ;get next VERTEX
              old e)                             ;keep previous VERTEX	    
        (if (cdr(assoc 10 (entget old)))         ;get last VERTEX point
         (setq pt_list (cons (cdr (assoc 10 (entget old))) pt_list))
        );end if
      )                                          ;end while
      (reverse pt_list)
    )                                            ;end progn
    (progn
     (prompt "\nSelected entity is NOT a POLYLINE\n")  ;if not pline tell me
     nil
    )
  )                                              ;end if
)                                                ;end pl_last
(defun poly_error (msg)
; This is a generic error trapping routine.
; It suppresses the quit, function cancelled, and console break messages

 (if (member msg '("Function cancelled" "console break" "quit / exit abort"))
  (princ)
  (princ (strcat "\nError: " msg))
 ); end if
 (setq *error* save-error
       save-error nil)
 (princ)
)
(princ)
